package io.gitlab.jfronny.breakme;

import io.gitlab.jfronny.breakme.config.Cfg;
import io.gitlab.jfronny.breakme.config.CrashCause;
import io.gitlab.jfronny.breakme.crash.CrashProvider;
import me.shedaniel.autoconfig.AutoConfig;
import me.shedaniel.autoconfig.serializer.JanksonConfigSerializer;
import net.fabricmc.api.ModInitializer;
import net.minecraft.entity.player.PlayerEntity;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BreakMe implements ModInitializer {
    public static final String MOD_ID = "breakme";
    private static final Logger log = LogManager.getFormatterLogger(MOD_ID);
    public static Cfg cfg;
    private static final Map<String, CrashProvider> crashProviders;

    static {
        //Get config
        AutoConfig.register(Cfg.class, JanksonConfigSerializer::new);
        cfg = AutoConfig.getConfigHolder(Cfg.class).getConfig();
        //Get crash providers
        crashProviders = new HashMap<>();
        try {
            List<Class<?>> classes = ClassFinder.find(CrashProvider.class.getPackage().getName());
            for (Class<?> clazz : classes) {
                if (CrashProvider.class.isAssignableFrom(clazz) && !CrashProvider.class.equals(clazz)) {
                    try {
                        CrashProvider provider = (CrashProvider) clazz.getDeclaredConstructor().newInstance();
                        crashProviders.put(provider.getName(), provider);
                    } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
                        e.printStackTrace();
                    }
                }
            }
            if (!crashProviders.containsKey(cfg.method)) {
                cfg.method = "None";
                Log("Could not find specified crash provider, defaulting to None");
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void Log(String str) {
        log.log(Level.INFO, "[" + MOD_ID + "] " + str);
    }

    @Override
    public void onInitialize() {
        Log("Prepare for trouble");
    }

    public static void Crash(PlayerEntity player) throws Exception {
        if (cfg.event == CrashCause.All || cfg.event == CrashCause.Damage || (cfg.event == CrashCause.Death && player.isDead())) {
            Log("invoking the crash");
            if (!crashProviders.containsKey(cfg.method)) {
                cfg.method = "None";
                Log("Could not find specified crash provider, defaulting to None");
            }
            crashProviders.get(cfg.method).crash();
        }
    }

    public static String[] getProviders() {
        return crashProviders.keySet().toArray(new String[0]);
    }
}
